#!/usr/bin/python
#
# simple json to rss convertor for lemonde.fr API.
# Reversed based on informations found here:
# https://apkscan.nviso.be/report/show/179338fd186d6e83e32262ef8100642c
#
# (c) 2016-2017 Arnaud Patard <apatard@hupstream.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#   MA 02111-1307 USA

import os
import os.path
import sys
import pprint
import json
import cgi
import time
from threading import Thread
from SocketServer import ThreadingMixIn

try:
    from http import server
except ImportError:
    import BaseHTTPServer as server

try:
  import http.client as client
except ImportError:
  import httplib as client

baseuri="www.lemonde.fr"
confreq="/export/1/android-phone/configuration.json"
#basereq="/export/android/sequence/"
#basereq="/export/1/android-phone/gratuit/rubrique/"

def get_feed_list():
#  with open('configuration.json','r') as infile:
#      conf=json.load(infile)
  try:
    conn = client.HTTPSConnection(baseuri)
    conn.request('GET', confreq)
    response = conn.getresponse()
    ans = response.read().decode('utf-8')
    conf = json.loads(ans)
    conn.close()
  except:
    return []

  b = conf['baseURL']['default'].replace('http://www.lemonde.fr','')
  r = [ conf['home']['default'] ]
  r += conf['rubrics']
  return (b, r)

def get_outline(r):
  buf = "\t<outline text=\"%s\" xmlUrl=\"%s/%s\" />\n" % (r['title'].encode('utf-8'), 'localhost', r['id'].encode('utf-8'))
  return buf

def get_opml(rubrics):
  buf = "<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n"
  buf += "<opml>\n"
  buf += "<head><title>lemonde.fr</title></head>\n"
  buf += "<body>\n"
  buf += '<outline text="Le Monde">\n'
  for r in rubrics:
    buf += get_outline(r)
  buf += "</outline>\n"
  buf += "</body>\n"
  buf += "</opml>"
  return buf

def get_feed(basereq, rubric):
  uri = "%s%s" % (basereq, rubric['path'])
  try:
    conn = client.HTTPSConnection(baseuri)
    conn.request('GET', uri)
    response = conn.getresponse()
    _ans = response.read().decode('utf-8')
    ans = json.loads(_ans)
    conn.close()
  except:
    return []

  return ans

def key2xml(key):
  trans = {
    'titre_appel': 'title',
    'date_publication': 'pubDate',
    'contenu': 'description',
    'lien_web': 'link',
    'site_web': None,
    'auteur': 'author',
    'tags': 'category',
    'tags_categorie': None, # ignore for now
    'nb_commentaires': None,
    'lien': None,
    'type_nom': None,
    'date_premiere_publication': None,
    'date_fin_avant_premiere': None,
    'exclusif': None,
    'restreint': None,
    'images': 'images',
    'id': None,
    'sequence': None,
    'sequence_id': None,
    'ensemble': None,
    'dernier_commentaire': None,
    'date_commentaire': None,
    'auteur_commentaire': None,
    'nb_envois': None,
    'nature_edito': None,
    'url_commentaires': 'comments',
    'contenu_percent': None,
    'atomes': None,
    'description_appel': None,
    'titre_court': None,
    'titre_une': None,
    'type_sub': None,
    'imagesPortfolio': None,
    'sonPath': None,
  }
  if key not in trans.keys():
    raise NameError("unhandled key %s" % key)
  return trans[key]

def get_channel(basereq, rubric):
  buf = ""
  feed = get_feed(basereq, rubric);
#  with open(rubric['id'], 'w') as outfile:
#      json.dump(feed, outfile)

  #with open('feed.json','r') as infile:
  #    feed=json.load(infile)

  if 'titre' in feed.keys():
    titre = feed['titre'].encode('utf-8')
  else:
    titre = rubric['title'].encode('utf-8')
  buf += "<channel><title>LeMonde.fr - %s</title>" % titre
  buf += "<pubDate>%s</pubDate>" % (feed['date_generation'].encode('utf-8'))
  if 'description' in feed.keys() and feed['description'] != "":
    buf += "<description>%s</description>" % (cgi.escape(feed['description'].encode('utf-8')))
  for i in feed['items']:
    buf += "<item>"
    for key in i.keys():
      k = key2xml(key)
      if k is not None and i[key] is not None:
        if k == 'category':
          for c in i[key]:
           buf += "<category>%s</category>" % (cgi.escape(c['nom']).encode('utf-8'))
        elif k == 'images':
          for img in i[key]:
            if img['format'] == 'petit':
              buf += '<enclosure url="%s" type="%s"/>' % (img['url'].encode('utf-8'),img['type'].encode('utf-8'))
        elif k == 'comments':
          buf += "<comments>http://%s/%s</comments>" % (baseuri, cgi.escape(i[key]).encode('utf-8'))
        else:
          buf += "<%s>%s</%s>" % (k.encode('utf-8'), cgi.escape(i[key]).encode('utf-8'), k.encode('utf-8'))
    buf += "</item>"
  buf += "</channel>"
  return buf

def fetch_feeds(basereq, rubrics):
  buf='<?xml version="1.0" encoding="utf-8"?><rss version="2.0">'
  for r in rubrics:
    buf += get_channel(basereq, r)
  buf += "</rss>"
  return buf

def fetch_feed(basereq, rubric):
  buf='<?xml version="1.0" encoding="utf-8"?><rss version="2.0">'
  buf += get_channel(basereq, rubric)
  buf += "</rss>"
  return buf


class HTTPRequestHandler(server.BaseHTTPRequestHandler):
  protocol_version = 'HTTP/1.1'
  server_version = "plop/0.1"

  def send_data(self, data):
    self.send_response(200)
    self.send_header('Content-Length', len(data))
    self.end_headers()
    self.wfile.write(data)
    self.wfile.flush()

  def do_GET(self):
      (basereq, rubrics) = get_feed_list()
      if self.path[:5] == '/opml':
          data = get_opml(rubrics)
          self.send_data(data)
          return

      for r in rubrics:
          if self.path[1:len(r['id'])+1] == r['id']:
              data=fetch_feed(basereq, r)
              self.send_data(data)
              return
      self.send_response(404)
      self.send_header('Content-Length', 0)
      self.end_headers()
      return
  def log_message(self, format, *args):
    return

class HTTPServer(server.HTTPServer):
  PROTOCOL = "http"
  def __init__(self, address, handler):
    server.HTTPServer.__init__(self, address, handler)

def serve(port):
    try:
      server = HTTPServer(('127.0.0.1', port), HTTPRequestHandler)
      server.serve_forever()
    except KeyboardInterrupt:
        server.socket.close()


if __name__ == '__main__':
   serve(9090)
#  (b, rubrics) = get_feed_list()
#  buf = get_opml(rubrics)
#  buf = fetch_feeds(b, rubrics)
#  print buf
